!MNH_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!--------------- special set of characters for RCS information
!-----------------------------------------------------------------
! $Source$ $Revision$
! MASDEV4_7 prep_ideal 2006/05/18 13:07:25
!-----------------------------------------------------------------
!     #########################
      MODULE MODI_DEFAULT_EXPRE
!     #########################
!
INTERFACE
!
SUBROUTINE DEFAULT_EXPRE
END SUBROUTINE DEFAULT_EXPRE
!
END INTERFACE
!
END MODULE MODI_DEFAULT_EXPRE
!
!
!
!     ########################
      SUBROUTINE DEFAULT_EXPRE
!     ########################
!
!!****  *DEFAULT_EXPRE * - set default values to variables in EXPRE file
!!
!!    PURPOSE
!!    -------
!       The purpose of this routine is to set default values for the variables
!     in EXPRE file by filling the corresponding variables 
!     stored in modules. These variables  will be stored in LFIFM file, and 
!     therefore they have not been set to default values by 
!     DEFAULT_DESFM.
!       
!
!!**  METHOD
!!    ------
!!      Every variable in modules, which can be initialized by  reading its 
!!    value in the EXPRE file is set to a default value.    
!!
!!     - Dimensions of the initial file which will be generated by the 
!!     preinitialization : NIMAX=10,NJMAX=10,NKMAX=10.
!!     - Geometry type used to generate the  initial file :
!!                         LCARTESIAN =.TRUE.
!!     - Conformal projection parameters used to generate the  initial file :
!!           XLON0=0.,XLAT0=60.,XBETA=0.,XRPK=1.,XLONORI=350.,XLATORI=37.
!!       (all longitudes and latitudes are in degrees)
!!
!!    EXTERNAL
!!    --------
!!      none
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------ 
!!
!!      Module MODD_CONF      : contains the declaration of configuration
!!                              variables for all models
!!         LCARTESIAN : Logical for cartesian geometry 
!!
!!
!!      Module MODD_DIM1      : contains dimension of one nested model
!!
!!         NIMAX : Dimensions in x direction 
!!         NJMAX : Dimensions in y direction                          
!!         NKMAX : Dimensions in z direction 
!!
!!      Module MODD_GRID      : contains grid variables for all models
!!
!!         XLON0 : Reference longitude for the conformal projection
!!         XLAT0 : Reference latitude for the conformal projection
!!         XBETA : Rotation angle for the conformal projection
!!         XRPK  : Projection parameter for the conformal projection
!!
!!        XLONORI : Longitude of the Origin point for the conformal projection
!!        XLATORI : Latitude of the Origin point for the conformal projection
!!
!!
!!    REFERENCE
!!    ---------
!!      
!!
!!    AUTHOR
!!    ------
!!	V. Ducrocq       * Meteo France *
!!
!!    MODIFICATIONS
!!    -------------
!!      Original                                           24/08/94 
!!      Namelist changes                                   26/10/94  (J.Stein)
!!      add the uniform soil values                        05/02/96  (J.Stein)
!!      removes default values for ground variables        26/11/96  (V.Masson)
!!      add default value for LBOUSS                       11/07/13  (C.Lac)     
!!      add default value LOCEAN LCOUPLES                    /03/21  (JL Redelsperger)
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
USE MODD_CONF           ! declarative modules
USE MODD_DIM_n
USE MODD_GRID
USE MODD_REF
USE MODD_DYN_n, ONLY : LOCEAN
!
IMPLICIT NONE
!
!-------------------------------------------------------------------------------
!
!*       1.    SET DEFAULT VALUES FOR MODD_CONF :
!              --------------------------------
!
LCARTESIAN = .TRUE.
!
!-------------------------------------------------------------------------------
!
!*       2.    SET DEFAULT VALUES FOR MODD_DIM1 :
!              ---------------------------------
!
NIMAX = 10
NJMAX = 10
NKMAX = 10
!
!-------------------------------------------------------------------------------
!
!*      3.    SET DEFAULT VALUES FOR MODD_GRID :
!             ----------------------------------
!
XLON0 = 0.
XLAT0 = 60.
XBETA = 0.
XRPK  = 1.
!
XLONORI = 350.
XLATORI = 37.
!
!-------------------------------------------------------------------------------
!
!*       4.    SET DEFAULT VALUES FOR MODD_REF :
!              --------------------------------
!
LBOUSS = .FALSE.
LOCEAN = .FALSE.
LCOUPLES= .FALSE.
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE DEFAULT_EXPRE
